import { defineStore } from "pinia";
import type {
  RespiratoryWearerViewModel,
  CreateRespiratoryWearerViewModel,
  UpdateRespiratoryWearerViewModel,
} from "@/viewmodels/admin/unit/respiratoryWearer/respiratoryWearer.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useRespiratoryWearerStore = defineStore("respiratoryWearer", {
  state: () => {
    return {
      respiratoryWearers: [] as Array<RespiratoryWearerViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
      activeRespiratoryWearer: null as string | null,
      activeRespiratoryWearerObj: null as RespiratoryWearerViewModel | null,
      loadingActive: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchRespiratoryWearers(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.respiratoryWearers = [];
      this.loading = "loading";
      http
        .get(`/admin/respiratoryWearer?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.respiratoryWearers
            .filter(
              (elem: RespiratoryWearerViewModel) => this.respiratoryWearers.findIndex((m) => m.id == elem.id) == -1
            )
            .map(
              (elem: RespiratoryWearerViewModel, index: number): RespiratoryWearerViewModel & { tab_pos: number } => {
                return {
                  ...elem,
                  tab_pos: index + offset,
                };
              }
            )
            .forEach((elem: RespiratoryWearerViewModel & { tab_pos: number }) => {
              this.respiratoryWearers.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllRespiratoryWearers(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/respiratoryWearer?noLimit=true`).then((res) => {
        return { ...res, data: res.data.respiratoryWearers };
      });
    },
    async getRespiratoryWearersByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/respiratoryWearer/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.respiratoryWearers };
        });
    },
    async searchRespiratoryWearers(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/respiratoryWearer?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.respiratoryWearers };
      });
    },
    fetchRespiratoryWearerByActiveId() {
      this.loadingActive = "loading";
      http
        .get(`/admin/respiratoryWearer/${this.activeRespiratoryWearer}`)
        .then((res) => {
          this.activeRespiratoryWearerObj = res.data;
          this.loadingActive = "fetched";
        })
        .catch((err) => {
          this.loadingActive = "failed";
        });
    },
    fetchRespiratoryWearerById(id: string) {
      return http.get(`/admin/respiratoryWearer/${id}`);
    },
    async printRespiratoryWearerByActiveId() {
      return http.get(`/admin/respiratoryWearer/${this.activeRespiratoryWearer}/print`, {
        responseType: "blob",
      });
    },
    fetchRespiratoryWearerStatisticsById(id: string) {
      return http.get(`/admin/respiratoryWearer/${id}/statistics`);
    },
    async createRespiratoryWearer(
      respiratoryWearer: CreateRespiratoryWearerViewModel
    ): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/respiratoryWearer`, {
        salutationId: respiratoryWearer.salutationId,
        firstname: respiratoryWearer.firstname,
        lastname: respiratoryWearer.lastname,
        nameaffix: respiratoryWearer.nameaffix,
        birthdate: respiratoryWearer.birthdate,
        internalId: respiratoryWearer.internalId,
      });
      this.fetchRespiratoryWearers();
      return result;
    },
    async updateActiveRespiratoryWearer(
      respiratoryWearer: UpdateRespiratoryWearerViewModel
    ): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/respiratoryWearer/${respiratoryWearer.id}`, {
        salutationId: respiratoryWearer.salutationId,
        firstname: respiratoryWearer.firstname,
        lastname: respiratoryWearer.lastname,
        nameaffix: respiratoryWearer.nameaffix,
        birthdate: respiratoryWearer.birthdate,
        internalId: respiratoryWearer.internalId,
      });
      this.fetchRespiratoryWearers();
      return result;
    },
    async deleteRespiratoryWearer(respiratoryWearer: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/respiratoryWearer/${respiratoryWearer}`);
      this.fetchRespiratoryWearers();
      return result;
    },
  },
});
